Imports System.Xml.Serialization
Imports System.IO

' A bouncing object.
Public Class Bouncer

    Public Enum BouncerTypeEnum
        bouncer_Circle
        bouncer_Diamond
        bouncer_Square
    End Enum

    ' Public properties.
    Public BouncerType As BouncerTypeEnum
    Public Radius As Integer
    Public X As Integer
    Public Y As Integer
    Public dX As Integer
    Public dY As Integer
    Public ColorName As String

    ' Initialize the color so we're sure it gets set.
    Public Sub New()
        ColorName = "black"
    End Sub

    ' Make a new random Bouncer.
    Public Shared Function RandomBouncer(ByVal min_radius As Integer, ByVal max_radius As Integer, ByVal xmax As Integer, ByVal ymax As Integer, ByVal dxmin As Integer, ByVal dxmax As Integer) As Bouncer
        Dim rand As New Random()
        Dim new_bouncer As Bouncer

        new_bouncer = New Bouncer()
        new_bouncer.X = rand.Next(min_radius, xmax - min_radius + 1)
        new_bouncer.Y = rand.Next(min_radius, ymax - min_radius + 1)
        new_bouncer.dX = rand.Next(dxmin, dxmax + 1)
        If rand.Next(2) = 0 Then new_bouncer.dX = -new_bouncer.dX
        new_bouncer.dY = rand.Next(dxmin, dxmax + 1)
        If rand.Next(2) = 0 Then new_bouncer.dY = -new_bouncer.dY
        new_bouncer.Radius = rand.Next(min_radius, max_radius + 1)
        Select Case rand.Next(3)
            Case 0
                new_bouncer.BouncerType = Bouncer.BouncerTypeEnum.bouncer_Circle
            Case 1
                new_bouncer.BouncerType = Bouncer.BouncerTypeEnum.bouncer_Diamond
            Case 2
                new_bouncer.BouncerType = Bouncer.BouncerTypeEnum.bouncer_Square
        End Select
        Select Case rand.Next(3)
            Case 0
                new_bouncer.ColorName = "red"
            Case 1
                new_bouncer.ColorName = "green"
            Case 2
                new_bouncer.ColorName = "blue"
        End Select

        ' Save the new Bouncer into the database.
        new_bouncer.SaveBouncer()

        ' Return the new Bouncer.
        Return new_bouncer
    End Function

    ' Draw the object.
    Public Sub Draw(ByVal gr As Graphics)
        Dim drawing_color As Color = Color.FromName(ColorName)
        Dim the_pen As New Pen(drawing_color)
        Dim the_brush As Brush = New SolidBrush(drawing_color)

        Select Case BouncerType
            Case BouncerTypeEnum.bouncer_Circle
                gr.FillEllipse(the_brush, X - Radius, Y - Radius, 2 * Radius, 2 * Radius)
                gr.DrawEllipse(the_pen, X - Radius, Y - Radius, 2 * Radius, 2 * Radius)
            Case BouncerTypeEnum.bouncer_Diamond
                Dim pts(3) As Point
                pts(0).X = X - Radius
                pts(0).Y = Y
                pts(1).X = X
                pts(1).Y = Y - Radius
                pts(2).X = X + Radius
                pts(2).Y = Y
                pts(3).X = X
                pts(3).Y = Y + Radius

                gr.FillPolygon(the_brush, pts)
                gr.DrawPolygon(the_pen, pts)
            Case BouncerTypeEnum.bouncer_Square
                gr.FillRectangle(the_brush, X - Radius, Y - Radius, 2 * Radius, 2 * Radius)
                gr.DrawRectangle(the_pen, X - Radius, Y - Radius, 2 * Radius, 2 * Radius)
        End Select
    End Sub

    ' Move the object, keeping it in bounds.
    Public Sub Move(ByVal xmax As Integer, ByVal ymax As Integer)
        X = X + dX
        If X > xmax - Radius Then
            X = (xmax - Radius) - (X - (xmax - Radius))
            dX = -dX
        End If
        If X < Radius Then
            X = Radius + (Radius - X)
            dX = -dX
        End If

        Y = Y + dY
        If Y > ymax - Radius Then
            Y = (ymax - Radius) - (Y - (ymax - Radius))
            dY = -dY
        End If
        If Y < Radius Then
            Y = Radius + (Radius - Y)
            dY = -dY
        End If
    End Sub

    ' Delete all Bouncers from the database.
    Public Shared Sub DeleteAllBouncers()
        ExecuteNonQuery("DELETE FROM Bouncers")
    End Sub

    ' Save the Bouncer into the database.
    Public Sub SaveBouncer()
        Dim xml_serializer As XmlSerializer
        Dim string_writer As New StringWriter()
        Dim serialization As String

        ' Get the Bouncer's serialization.
        xml_serializer = New XmlSerializer(GetType(Bouncer))
        xml_serializer.Serialize(string_writer, Me)
        serialization = string_writer.ToString

        ' Create the new Bouncers record.
        ExecuteNonQuery( _
            "INSERT INTO Bouncers (Serialization) VALUES ('" & _
            serialization & "')")
    End Sub

    ' Fetch the Bouncers that meet some condition.
    Public Shared Function FetchBouncers(ByVal condition As String) As Collection
        Dim bouncers As Collection
        Dim query As String
        Dim data_set As DataSet
        Dim new_bouncer As Bouncer
        Dim serialization As String
        Dim string_reader As StringReader
        Dim xml_serializer As New XmlSerializer(GetType(Bouncer))
        Dim i As Integer

        ' Start a new collection.
        bouncers = New Collection()

        ' Get the data.
        query = "SELECT Serialization " & _
            "FROM Bouncers " & condition
        data_set = ExecuteQuery(query)

        ' If we got nothing, do no more.
        If data_set.Tables(0).Rows.Count = 0 Then Return bouncers

        ' Deserialize the Bouncers.
        For i = 0 To data_set.Tables(0).Rows.Count - 1
            ' Deserialize the Bouncer.
            serialization = data_set.Tables(0).Rows(i).Item(0)
            string_reader = New StringReader(serialization)
            new_bouncer = xml_serializer.Deserialize(string_reader)

            bouncers.Add(new_bouncer)
        Next i

        Return bouncers
    End Function

End Class
